#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <time.h>

#include "protocollo.h"
#include "page_generator.h"

/* Riceve un comando come uint16
 * Restituisce un valore negativo se c'è un fallimento in ricezione.
 */
int leggiComando(int cn_fd) {
  uint16_t comando;
  int ret;
  // Ricevo un comando dal socket
  ret = recv(cn_fd, (void *) &comando, sizeof(comando), MSG_WAITALL);
  if (ret < 0 || ret < sizeof(comando)) { /* error */
      if (ret < 0) {
          perror("recv(len)");
      }
      return -1;
  }
  // Converto in formato locale
  comando = ntohs(comando);
  return comando;
}

/* Invia l'esito di una operazione codificato come uint16
 * Restituisce un valore negativo in caso di fallimento in trasmissione.
 */
int inviaEsito(int fd, uint16_t v) {
  uint16_t esito;
  int ret;

  // Converte in formato rete
  esito = htons(v);
  // Invia usando il socket fd
  ret = send(fd, (void *) &esito, sizeof(esito), 0);
  if (ret < 0 || ret < sizeof(esito)){  /* error */
    if (ret < 0) {
        perror("send");
    }
    return -1;
  }
  return ret;
}

int main (int argc, char *argv[]) {
    struct sockaddr_in my_addr, cl_addr;
    int ret, fd, cn_fd;
    socklen_t addrlen;
    uint16_t comando;
    char linea_di_comando[512];
    time_t t;
    struct tm tm;

    if (argc < 3) {
        printf("Wrong number of parameters!\n");
        printf("USAGE: %s [SERVER_IP] [PORT]\n", argv[0]);
        exit(1);
    }

    // Crea un socket TCP
    fd = socket(PF_INET, SOCK_STREAM, 0);
    // Azzera la struttura
    memset(&my_addr, 0, sizeof(my_addr));
    // Indirizzo Internet
    my_addr.sin_family = AF_INET;
    // Converte indirizzo IP da stringa
    inet_pton(AF_INET, argv[1], &my_addr.sin_addr);
    // Converte porta da stringa
    my_addr.sin_port = htons(atoi(argv[2]));

    // Esegue bind
    ret = bind(fd, (struct sockaddr *) &my_addr, sizeof(my_addr));
    if(ret < 0) {
      perror("bind");
      return ret;
    }
    // Si mette in ascolto
    ret = listen(fd, 10);
    if(ret < 0) {
      perror("listen");
      return ret;
    }

    // Ciclo infinito
    for (;;) {
        // Attende una connessione da parte di un client
        addrlen = sizeof(cl_addr);
        cn_fd = accept(fd, (struct sockaddr *) &cl_addr, &addrlen);
        printf("Client connesso\n");

        // Ciclo in cui si ricevono comandi dal client che si s' appena connesso
        do {
            // Legge un comando dal socket. Il comando e' codificato come un intero.
            comando = leggiComando(cn_fd);
            printf("Ho ricevuto un comando\n");

            // Esegue azione corrispondente al comando. Per semplicita' e'
            // implementato un solo comando (FOTO_COLORI) oltre a quello per
            // terminare l'interazione (FINE).
            switch (comando) {
              case FOTO_COLORI:
                // Il nome del file jpg della nuova foto e' generato a partire
                // dal istante corrente.
                // time restituisce il numero di secondi trascorsi dall'epoch
                t = time(NULL);
                // Converte in formato data + ora + minuti etc
                tm = *localtime(&t);
                // L'acquisizione di una nuova foto avvine e attraverso l'eseguibile
                // raspistill (per semplicita'). Il paramtro -q indica il livello di qualita' della
                // foto, -n disabilita la preview, -o il nome del file dell'immagine.
                sprintf(linea_di_comando, "raspistill -q 5 -n -o /home/pi/public_html/picture-%d-%d-%d-%d-%d-%d.jpg",
                                          tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday,
                                          tm.tm_hour, tm.tm_min, tm.tm_sec);
                // Esegue la riga di comando
                system(linea_di_comando);
                // Crea una nuova pagina index.html che contiene i link alle
                // immagini (al piu' 10).
                produciPagina("/home/pi/public_html/");
                // Invia l'esito al client. Bisognerebbe controllare che tutto
                // sia andato effettivamente bene...
                inviaEsito(cn_fd, OK);
                break;
              case FINE:
                // Il client vuole uscire.
                printf("Client ha terminato.\n");
                inviaEsito(cn_fd, OK);
            }
        } while (comando != FINE);
        // Client uscito. Si rimette in ascolto.
        close(cn_fd);
    }

    return 0;
}
